'''
Created on 20 oct. 2012

@author: coissac
'''

from os import path
import os.path
import glob

from distutils.core import setup as ori_setup
from distutils.extension import Extension


from obidistutils.command.build import build
from obidistutils.command.build_cexe import build_cexe
from obidistutils.command.build_ext import build_ext
from obidistutils.command.build_ctools import build_ctools
from distutils.command.build_scripts import build_scripts
from obidistutils.command.install_scripts import install_scripts
from obidistutils.dist import Distribution
import sys

try:
    from Cython.Compiler import Main
    has_cython=True
except ImportError:
    has_cython=False

def findPackage(root,base=None):
    modules=[]
    if base is None:
        base=[]
    for module in (path.basename(path.dirname(x)) 
                   for x in glob.glob(path.join(root,'*','__init__.py'))):
        modules.append('.'.join(base+[module]))
        modules.extend(findPackage(path.join(root,module),base+[module]))
    return modules
    
def findCython(root,base=None,pyrexs=None):
    setupdir = os.path.dirname(sys.argv[0])
    modules=[]
    pyrexs=[]
    #o=dict(Main.default_options)
    pyopt=Main.CompilationOptions(Main.default_options)
    #print dir(pyopt)
    Main.__dict__['context'] = Main.Context(pyopt.include_path, {})
    if base is None:
        base=[]
    for module in (path.basename(path.dirname(x)) 
                   for x in glob.glob(path.join(root,'*','__init__.py'))):
                       
                
        for pyrex in glob.glob(path.join(root,module,'*.pyx')):
            pyrexs.append(Extension('.'.join(base+[module,path.splitext(path.basename(pyrex))[0]]),[pyrex]))
            try:
                cfiles = os.path.splitext(pyrex)[0]+".cfiles"
                cfilesdir = os.path.dirname(cfiles)
                cfiles = open(cfiles)
                cfiles = [os.path.relpath(os.path.join(cfilesdir,y),setupdir).strip() for y in cfiles]
                incdir = set(os.path.dirname(x) for x in cfiles if x[-2:]==".h")
                cfiles = [x for x in cfiles if x[-2:]==".c"]                
                pyrexs[-1].sources.extend(cfiles)
                pyrexs[-1].include_dirs.extend(incdir)
                pyrexs[-1].define_macros.extend([('LITTLE_END',None),('__SSE2__',None)])
                
            except IOError:
                pass
            pyrexs[-1].sources.extend(glob.glob(os.path.splitext(pyrex)[0]+'.ext.*.c'))
            print pyrexs[-1].sources
            Main.compile([pyrex],timestamps=True,recursion=True)
            
        pyrexs.extend(findCython(path.join(root,module),base+[module]))
    return pyrexs
    
def findC(root,base=None,pyrexs=None):
    setupdir = os.path.dirname(sys.argv[0])
    modules=[]
    pyrexs=[]
    if base is None:
        base=[]
    for module in (path.basename(path.dirname(x)) 
                   for x in glob.glob(path.join(root,'*','__init__.py'))):
                
        for pyrex in glob.glob(path.join(root,module,'*.c')):
            if '.ext.' not in pyrex:
                pyrexs.append(Extension('.'.join(base+[module,path.splitext(path.basename(pyrex))[0]]),[pyrex]))
                try:
                    cfiles = os.path.splitext(pyrex)[0]+".cfiles"
                    cfilesdir = os.path.dirname(cfiles)
                    cfiles = open(cfiles)
                    cfiles = [os.path.relpath(os.path.join(cfilesdir,y),setupdir).strip() for y in cfiles]
                    incdir = set(os.path.dirname(x) for x in cfiles if x[-2:]==".h")
                    cfiles = [x for x in cfiles if x[-2:]==".c"]                
                    pyrexs[-1].sources.extend(cfiles)
                    pyrexs[-1].include_dirs.extend(incdir)
                except IOError:
                    pass
                pyrexs[-1].sources.extend(glob.glob(os.path.splitext(pyrex)[0]+'.ext.*.c'))
                print pyrexs[-1].sources
       
        pyrexs.extend(findC(path.join(root,module),base+[module]))
    return pyrexs

def rootname(x):
    return os.path.splitext(x.sources[0])[0]

COMMANDS = {'build':build,
            'build_cexe':build_cexe, 
            'build_ctools':build_ctools, 
            'build_ext': build_ext,
            'build_scripts':build_scripts, 
            'install_scripts':install_scripts}

CTOOLS =[]
CEXES  =[]


def setup(**attrs):

    if 'distclass' not in attrs:
        attrs['distclass']=Distribution

    if 'python_src' not in attrs:
        SRC = 'src'
    else:
        SRC = attrs['python_src']
        del(attrs['python_src'])
    
    if 'scripts' not in attrs:
        attrs['scripts'] = glob.glob('%s/*.py' % SRC)

    if 'package_dir' not in attrs:
        attrs['package_dir'] = {'': SRC}

    if 'packages' not in attrs:
        attrs['packages'] = findPackage(SRC)

    
    if 'cmdclass' not in attrs:
        attrs['cmdclass'] = COMMANDS

    if 'requires' not in attrs:
        attrs['requires'] = ['Cython (>=0.17)']
    else:
        attrs['requires'].append('Cython (>=0.17)')

    if 'ctools' not in attrs:
        attrs['ctools'] = CTOOLS
    
    if 'executables' not in attrs:
        attrs['executables'] = CEXES
    

    if  has_cython:
        EXTENTION=findCython(SRC)
        CEXTENTION=findC(SRC)
        cython_ext = set(rootname(x) for x in EXTENTION)
        EXTENTION.extend(x for x in CEXTENTION if rootname(x) not in cython_ext)
    else:
        EXTENTION=findC(SRC)
    
    if 'ext_modules' not in attrs:
        attrs['ext_modules'] = EXTENTION
    
    ori_setup(**attrs)
