#!/usr/local/bin/python
################################################
# setenv PYTHONPATH ./orgasm/build/lib.macosx-10.6-intel-2.7/
# echo $PYTHONPATH
# ./orgasm/build/lib.macosx-10.6-intel-2.7/
# python orgasm-1.py GWM-261 COX1.fasta

from orgasm.tango import tango, matchtoseed, cutLowCoverage, scaffold, coverageEstimate
from orgasm.indexer import _orgasm as indexer
from orgasm.backtranslate.fasta import fasta
from orgasm.assembler import Assembler

import sys

if __name__ == '__main__':
    
    import argparse
    
    parser = argparse.ArgumentParser(description='Assembly program dedicated to organel genome reconstruction.')
    parser.add_argument('indexFilename', metavar='index', help='index root filename (produced by orgasmi)')
    
    parser.add_argument('seedsFilenameAA', metavar='seeds', help='fasta file containing seeds proteic sequences')
    
    parser.add_argument('--minread', dest='minread', type=int, action='store', default=20, help='the minimum count of read to consider [default: %(default)d]')
    parser.add_argument('--minratio', dest='minratio', type=float, action='store', default=0.1, help='minimum ratio between occurrences of an extension and the occurrences of the most frequent extension to keep it. [default: %(default)f]')
    parser.add_argument('--mincov', dest='mincov', type=int, action='store', default=2, help='minimum occurrences of an extension to keep it. [default: %(default)d]')
    parser.add_argument('--minoverlap', dest='minoverlap', type=int, action='store', default=40, help='minimum length of the overlap between the sequence and reads to participate in the extension. [default: %(default)d]')
    parser.add_argument('--lowcomplexity', dest='lowcomplexity', action='store_true', default=False, help='Use also low complexity probes')
    parser.add_argument('--maxjump', dest='maxjump', type=int, action='store', default=1, help='COunt of pair end jump by extension branch')
    
    
    args = parser.parse_args()
    
    

    r = indexer.Index(args.indexFilename)
    
    

    p = fasta(args.seedsFilenameAA)
    
    x = r.lookForSeeds(p)
    
    s = matchtoseed(x,r)
    

    asm = Assembler(r)
    
    a = tango(asm,s,mincov=args.mincov,
                    minread=args.minread,
                    minoverlap=args.minoverlap,
                    lowfilter=not args.lowcomplexity,
                    maxjump=args.maxjump)
    
    
    asm.cleanDeadBranches()
    cutLowCoverage(asm,args.mincov)
    cutLowCoverage(asm,args.mincov,False)
    sg = asm.compactAssembling()
    
    scaffold(sg, r)


    print>>sys.stdout, sg.gml()

