from cpython.bytes cimport PyBytes_FromStringAndSize, PyBytes_GET_SIZE

#                   "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
cdef char* __COMP = "T-G---C------------A------"

cdef int _reverseComplement(char* seq, size_t length):
    """
    Internal function computing the reverse complement sequence
    of a DNA sequence in place.
    """
    cdef char   tmp
    cdef char*  end = seq + length -1
    
    while seq <= end:
        tmp = end[0]
        end[0] = __COMP[(seq[0] & (~32)) - 65]
        seq[0] = __COMP[(tmp & (~32)) - 65]
        seq+=1
        end-=1 
        
    return 0
    
cpdef bytes reverseComplement(bytes seq):
    """
    Computes the reverse complement sequence of a DNA sequence
    composed only of acgt. No check is done for efficiency 
    purpose so be careful to the input sequence.
    
    :param seq: the sequence to revert-compelent expressed as
                a serie of acgt in upper or lower case
    :type  seq: a bytes instance
    
    :return: the reverse-complemented sequence in upper case
    :rtype:  a bytes instance
    """
    cdef bytes  comp 
    cdef char*  ccomp= seq
    cdef size_t lcomp= PyBytes_GET_SIZE(seq)
    
    comp = PyBytes_FromStringAndSize(ccomp,lcomp)
    ccomp= comp
    
    _reverseComplement(ccomp,lcomp)
    
    return comp
